@@ -2,21 +2,20 @@ package DBIx::Simple::Procedure;
 
 use warnings;
 use strict;
-use base 'DBIx::Simple';
+use DBIx::Simple;
 
 =head1 NAME
 
-DBIx::Simple::Procedure - An Alternative To DB Stored Procedures using DBIx::Simple
+DBIx::Simple::Procedure - An Alternative To SQL Stored Procedures using DBIx::Simple
 
 =head1 VERSION
 
-Version 1.02
+Version 1.31
 
 =cut
 
-our $VERSION = '1.22';
+our $VERSION = '1.31';
 our @properties = caller();
-# depreciated [now sports a commands dispatch table] - our @sqlfile_commands = ('capture', 'include', 'execute', 'replace', 'storage', 'proceed');
 
 =head1 SYNOPSIS
 
@@ -25,7 +24,7 @@ is an alternative to database stored procedures and not a replacement or emulati
 
 Here is an example of how to setup and process a (sql) text file.
 
-    # DBIx::Simple::Procedure uses L<DBIx::Simple> as it base class and provides an accessor through the DBIx::Simple::Procedure->{dbix} hash reference.
+    # DBIx::Simple::Procedure uses DBIx::Simple and provides an accessor through the DBIx::Simple::Procedure->{dbix} hash reference.
     use DBIx::Simple::Procedure;
     my $db = DBIx::Simple::Procedure->new($path_to_sqlfiles, 'dbi:SQLite:dbname=file.dat'); # will error out using DBIx::Simple if a connection error occurs.
     
@@ -55,11 +54,13 @@ SQL File Commands:
     ! replace: This is an execute command that after successfully executed, replaces the scope parameters with data from the last row in its dataset. Note! This command can only be used with a select statement.
     ! include: This command processes the supplied sql file in a sub transaction. Note! Included sql file processing is isolated from the current processing. Any capture commands encountered in the included sql files will not cache the dataset.
     ! proceed: This command should be read "proceed if" because it evaluates the string passed (perl code) for truth, if true, it continues if false it skips to the next proceed command or until the end of the sql file.
+    ! ifvalid: This command is a synonym for proceed.
     ! storage: This command does absolutely nothing except store the sql statement in the commands list (queue) for processing individually from within the perl code with a method like process_command.
     ! declare: This command is effectively equivalent to the select .. into sql sytax and uses an sql select statement to add vairables to the scope for processing (e.g. ! declare select `name` from `foo` where `id` = $0)
                can be used in other instructions as $!name, e.g. ! execute update `foo` set `name` = $!name where `id` = $0.
     ! forward: This command takes an index and jumps to that command line and continues.
     ! process: This command takes an index and executes that command line.
+    ! examine: This command is used for debugging, it errors out with the compiled statement passed to it.
     
     Example: ! execute select * from `foo` where `id` = $0
 
@@ -132,48 +133,83 @@ sub _load_commands {
         $self->{processing}->{resultset} = $self->_execute_query($statement, @parameters);
         $self->{sets}->[@{$self->{sets}}] = $self->{processing}->{resultset}->hashes;
     };
+    
     #! execute: execute sql commands only, nothing else, nothing fancy
     $self->{commands}->{execute} = sub {
         my ($statement, @parameters) = @_;
         $self->{processing}->{resultset} = $self->_execute_query($statement, @parameters);
     };
-    #! skipif: evaluates the statement passed (perl code) for truth, if true, it continues if false it skips to the next proceed command or until the end of the sql file.
+    
+    #! proceed: evaluates the statement passed (perl code) for truth, if true, it continues if false it skips to the next proceed command or until the end of the sql file.
     $self->{commands}->{proceed} = sub {
         my ($statement, @parameters) = @_;
-        $statement =~ s/\?/$_/ foreach @parameters;
+        if (@parameters) {
+            foreach my $parameter (@parameters) {
+                $_ = '' if !$_;
+                $statement =~ s/\?/$_/;
+            }
+        }
         $self->{processing}->{skip_switch} = eval $statement ? 0 : 1;
     };
+    
+    #! ifvalid: a synonym for proceed
+    $self->{commands}->{ifvalid} = $self->{commands}->{proceed};
+    $self->{commands}->{validif} = $self->{commands}->{proceed};
+    
     #! replace: replaces parameters with the data from the last row of the resultset
     $self->{commands}->{replace} = sub {
         my ($statement, @parameters) = @_;
         $self->{processing}->{resultset} = $self->_execute_query($statement, @parameters);
         $self->{processing}->{parameters} = @{$self->{processing}->{resultset}->array};
     };
+    
     #! include: processes another (sql) text file
     $self->{commands}->{include} = sub {
         my ($statement, @parameters) = @_;
         my ($sub_sqlfile, $placeholders) = split /\s/, $statement;
         DBIx::Simple::Procedure->new($self->{path}, $self->{dbix}->{dbh})->queue($sub_sqlfile)->process_queue(@parameters);
     };
+    
     #! storage: stores sql statements for later
     $self->{commands}->{storage} = sub {
         my ($statement, @parameters) = @_;
     };
+    
     #! declare: uses an sql select statement to add vairables to the scope for processing
     $self->{commands}->{declare} = sub {
         my ($statement, @parameters) = @_;
         $self->{processing}->{resultset} = $self->_execute_query($statement, @parameters);
-        $self->{processing}->{custom_parameters} = $self->{processing}->{resultset}->hash;
+        my $results = $self->{processing}->{resultset}->hash;
+        if ($results) {
+            my %params = %{$results};
+            while ( my ($key, $val) = each %params ) {
+                $self->{processing}->{custom_parameters}->{$key} = $val;
+            }
+        }
     };
+    
     #! forward: changes the queue position, good for looping
     $self->{commands}->{forward} = sub {
         my ($statement, @parameters) = @_;
         $self->{cursor} = $statement;
     };
+    
+    #! process: executes a command in the queue by index number
     $self->{commands}->{process} = sub {
         my ($statement, @parameters) = @_;
         $self->process_command($statement, @parameters);
     };
+    
+    #! examine: dumps the passed sql statement to the screen (should not be left in the sql file)
+    $self->{commands}->{examine} = sub {
+        my ($statement, @parameters) = @_;
+        my $db = $self->{dbix}->{dbh};
+        foreach my $parameter (@parameters) {
+            my $placeholder = $db->quote($parameter);
+            $statement =~ s/\?/$placeholder/;
+        }
+        die $self->_error( $statement );
+    };
 }
 
 =head2 _execute_query
@@ -182,7 +218,24 @@ sub _load_commands {
 
 sub _execute_query {
     my ($self, $statement, @parameters) = @_;
-    return $self->{dbix}->query( $statement, @parameters ) or die ref($self) . " - sql file $self->{file} failed execution of command (capture) and statement (".substr($statement,0,10)."...) at $properties[1] on line $properties[2], " . $self->{dbix}->error;
+    my $resultset = $self->{dbix}->query( $statement, @parameters ) or die $self->_error(undef, @parameters);
+    return $resultset;
+}
+
+=head2 _error
+    The die method is an internal method that dies with a standardized error message.
+=cut
+
+sub _error {
+    my ($self, $message, @parameters) = @_;
+    return ref($self) .
+    " - sql file $self->{file} processing failed at the execution of command number " .
+    ( $self->{cursor} || '0') .
+    " [" . $self->{cmds}->[$self->{cursor}]->{command} . "] " .
+    ( $self->{cmds}->[$self->{cursor}]->{statement} ? ( "and statement (" . substr($self->{cmds}->[$self->{cursor}]->{statement}, 0, 20) . "...) " ) : " " ) .
+    ( @parameters ? ( "using " . join( ', ', @parameters ) . " " ) : "" ) . "at $properties[1]" .
+    " on line $properties[2], " .
+    ( $message || $self->{dbix}->error || "Check the sql file for errors" ) . ".";
 }
 
 =head2 queue
@@ -219,14 +272,14 @@ sub process_queue {
             if ( grep($cmd->{command} eq $_, keys %{$self->{commands}}) )
             {
                 # process command
-                $self->{cursor}++;
                 $self->_processor($cmd);
+                $self->{cursor}++;
             }
         }
         return $self->{processing}->{resultset};
     }
     else {
-        die ref($self) . " - Could'nt process sql file $self->{file} from $properties[1] on line $properties[2], file has no commands to process.";
+        die $self->_error( "File has no commands to process" );
     }
     return $self;
 }
@@ -256,21 +309,26 @@ sub _processor {
         # if the found param is a custom param
         if (defined $custom) {
             push @{$self->{processing}->{statement_parameters}}, $self->{processing}->{custom_parameters}->{$custom};
-            $statement =~ s/\$\![a-z0-9A-Z\_\-]+/\?/g;
+            $statement =~ s/\$\!$custom/\?/;
         }
         # if the found param is a passed-in param
         if (defined $passed) {
             push @{$self->{processing}->{statement_parameters}}, $self->{processing}->{parameters}[$passed];
-            $statement =~ s/\$\d+/\?/g;
+            $statement =~ s/\$$passed/\?/;
         }
     }
     
-    # skip command while skip_switch is turned on
-    next if $self->{processing}->{skip_switch} && $command ne "proceed";
-    
-    # execute command
-    $self->{commands}->{$command}->($statement, @{$self->{processing}->{statement_parameters}});
-    return $self->{processing}->{resultset};
+    if ($self->{processing}->{skip_switch} && $command ne "proceed")
+    {
+        # skip command while skip_switch is turned on
+        return;    
+    }
+    else
+    {
+        # execute command
+        $self->{commands}->{$command}->($statement, @{$self->{processing}->{statement_parameters}});
+        return $self->{processing}->{resultset};
+    }
 }
 
 =head2 cache
@@ -329,9 +387,9 @@ sub _parse_sqlfile {
     my (@lines, @statements);
     # open file and fetch commands
     $self->{file} = $sqlfile;
-    open (SQL, "$self->{path}$sqlfile") || die ref($self) . " - Could'nt open $self->{path}$sqlfile sql file from $properties[1] on line $properties[2], $!.";
+    open (SQL, "$self->{path}$sqlfile") || die $self->_error( "Could'nt open $self->{path}$sqlfile sql file" );
     push @lines, $_ while(<SQL>);
-    close SQL || die ref($self) . " - Could'nt close $self->{path}$sqlfile sql file from $properties[1] on line $properties[2], $!.";
+    close SQL || die $self->_error( "Could'nt close $self->{path}$sqlfile sql file" );
     # attempt to parse commands
     foreach my $command (@lines) {
         if ($command =~ /^\!/) {
@@ -356,7 +414,7 @@ sub _validate_sqlfile {
     foreach my $statement (@statements) {
         if (grep ( $statement->{command} eq $_, @{$self->{select_required}})) {
             if (lc($statement->{statement}) !~ /^(\s+)?select/) {
-                die ref($self) . " - Validation of the sql file $self->{file} failed within $properties[1] on line $properties[2]. The command ($statement->{command}) can only be used with a SQL (select) statement.";
+                die $self->_error( "Validation of the sql file $self->{file} failed. The command ($statement->{command}) can only be used with an SQL (select) statement.", $statement->{statement});
             }
         }
     }
